#!/usr/bin/env python

import numpy as np
from matplotlib import animation
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

from logslider import LogSlider

π = np.pi


def beatings(t, df, dφ, A=1., f=1.):
    ya = np.cos(2 * π * f * t)
    yb = A * np.cos(2 * π * ((f + df) * t + dφ / 2))
    return ya, yb, ya + yb


plt.style.use('dark_background')

fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True, sharey=True)

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Dudnienia")
manager.full_screen_toggle()

ax1.axhline(0, color='0.3', lw=0.7)
ax2.axhline(0, color='0.3', lw=0.7)

df = 0.0
dφ = 0.0

tt = np.linspace(0., 100., 10001)

ya, yb, y = beatings(tt, df, dφ)
line1a, = ax1.plot(tt, ya, lw=1.5, color='#81b1d2')
line1b, = ax1.plot(tt, yb, lw=1.5, color='#fa8174')
line2, = ax2.plot(tt, y, lw=2.0, color='#feffb3')

mark1a, = ax1.plot(tt[0], ya[0], 'o', color='#81b1d2', mec='0.3', lw=0.5)
mark1b, = ax1.plot(tt[0], yb[0], 'o', color='#fa8174', mec='0.3', lw=0.5)
mark2, = ax2.plot(tt[0], y[0], 'o', color='#bc82bd')

ax2.set_xlabel('$t$ [s]')
ax2.set_xlim(tt[0], tt[-1])

fig.tight_layout()

fig.subplots_adjust(bottom=0.22)
tax = fig.add_axes([0.25, 0.10, 0.65, 0.03])
max = fig.add_axes([0.25, 0.07, 0.65, 0.03])
fax = fig.add_axes([0.25, 0.04, 0.65, 0.03])
pax = fig.add_axes([0.25, 0.01, 0.25, 0.03])
aax = fig.add_axes([0.65, 0.01, 0.25, 0.03])

sst = {
    'track_color': '0.3',
    'color': '0.3',
    'initcolor': 'black',
}

time_slider = LogSlider(ax=tax, label='Czas całkowity [s]', valmin=1, valmax=3, valinit=2, valstep=0.333333, **sst)
mark_slider = Slider(ax=max, label='Znacznik [$T_\\mathsf{max}$]', valmin=0, valmax=1, valinit=0, **sst)
freq_slider = Slider(ax=fax, label='Różnica częstości [Hz]', valmin=-0.1, valmax=0.1, valstep=1e-3, valinit=df, **sst)
phas_slider = Slider(
    ax=pax, label='Przesunięcie fazowe $\\Delta\\phi$ [$\\pi$]', valmin=-1.0, valmax=1.0, valstep=0.125, valinit=dφ, **sst
)
ampl_slider = LogSlider(ax=aax, label='Stosunek amplitud', base=2, valmin=-2, valmax=0, valstep=1/3, valinit=0, **sst)


def on_slider(val):
    global tt
    tt = np.arange(0., time_slider.val + 0.005, 0.01)
    ax2.set_xlim(tt[0], tt[-1])
    ya, yb, y = beatings(tt, freq_slider.val, phas_slider.val, ampl_slider.val)
    line1a.set_data(tt, ya)
    line1b.set_data(tt, yb)
    line2.set_data(tt, y)
    t = mark_slider.val * tt[-1]
    ya, yb, y = beatings(t, freq_slider.val, phas_slider.val, ampl_slider.val)
    mark1a.set_data(t, ya)
    mark1b.set_data(t, yb)
    mark2.set_data(t, y)
    fig.canvas.draw_idle()


mark_slider.on_changed(on_slider)
time_slider.on_changed(on_slider)
freq_slider.on_changed(on_slider)
phas_slider.on_changed(on_slider)
ampl_slider.on_changed(on_slider)


def animate(i):
    i = (2 * i) % len(tt)
    t = tt[i]
    ya, yb, y = beatings(t, freq_slider.val, phas_slider.val, ampl_slider.val)
    mark1a.set_data(t, ya)
    mark1b.set_data(t, yb)
    mark2.set_data(t, y)
    return line1a, line1b, line2, mark1a, mark1b, mark2


# ani = animation.FuncAnimation(fig, animate, interval=10, blit=True)

plt.show()
